@@ -1,3 +1,4 @@
+import logging
 import requests
 import json
 from datetime import datetime
@@ -5,10 +6,13 @@ from datetime import datetime
 DC_PRODUCTION_URL = "https://dc.directchannel.it/mentor/api"
 DC_TEST_URL = "http://test.directchannel.it/mentor/api"
 
+_logger = logging.getLogger(__name__)
 
 class RequestWrapper(object):
-
     def _send_request(self, url, payload, headers, method="POST", debug=False):
+        if self.dryrun:
+            return {'result': 'OK', 'message': 'DRYRUN', 'data': '', 'payload': payload}
+
         try:
             response = requests.request(method, url=url, data=json.dumps(payload, ensure_ascii=False).encode('utf-8'), headers=headers)
             if debug:
@@ -19,19 +23,25 @@ class RequestWrapper(object):
                 message += "Response: %s\n" % response.text
                 with open(debug, 'a+') as f:
                     f.write(message)
-            return response.json()
+            data = response.json()
+            data['payload'] = payload
+            return data
         except requests.exceptions.RequestException as e:
             return {
                 "result": "KO",
                 "message": "SERVER ERROR {0}".format(str(e)),
                 "data": "",
+                "payload": payload,
             }
 
     def _handle_data(self, data):
         if hasattr(data, 'text'):
             return json.loads(data.text)
         if isinstance(data, dict) and data.get('data', False) and isinstance(data['data'], str):
-            data['data'] = json.loads(data['data'])
+            try:
+                data['data'] = json.loads(data['data'])
+            except Exception as e: 
+                _logger.warning("Cannot parse data '%s': %s" % (data['data'], e))
             return data
         else:
             return data
@@ -44,13 +54,14 @@ class DirectChannel(RequestWrapper):
     token = None
     user = None
 
-    def __init__(self, env, application, token, user, test=False, debug=False):
+    def __init__(self, env, application, token, user, test=False, debug=False, dryrun=False):
         self.url = DC_PRODUCTION_URL if not test else DC_TEST_URL
         self.env = env
         self.application = application
         self.token = token
         self.user = user
         self.debug = debug
+        self.dryrun = dryrun
 
     def wsc_table(self, param):
         payload = {
@@ -188,7 +199,7 @@ class DirectChannel(RequestWrapper):
                          token=None, mesetoken=None, providerincasso=None, nometitolare=None, datarichiesta=None, dataprossimarichiesta=None,
                          codicefiscaletitolare=None, indirizzotitolare=None, localitatitolare=None, cartaprepagata=None,
                          provinciatitolare=None, annotoken=None, cap=None, codicedialogatoreinterno=None, codicedialogatoreesterno=None, nomedialogatoreesterno=None,
-                         flagallineamento=None, datapromessa=None):
+                         flagallineamento=None, datapromessa=None, codiceagenzia=None):
         payload = {
             "env": self.env,
             "application": self.application,
@@ -206,6 +217,7 @@ class DirectChannel(RequestWrapper):
                 "codiceTema": codicetema,
                 "codiceProgetto": codiceprogetto,
                 "codiceCanale": codicecanale,
+                "codiceAgenzia": codiceagenzia,
                 "importo": importo,
                 "frequenza": frequenza,
                 "metodo": metodo,
